import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_custom_tabs/flutter_custom_tabs.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:intl/intl.dart';
import 'package:loading_overlay/loading_overlay.dart';
import 'package:pea_cremation/shares/app_drawer.dart';
import 'package:pea_cremation/domain/model/news.dart';
import 'package:pea_cremation/pages/news/news_page_view_model.dart';
import 'package:provider/provider.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

class NewsPage extends StatefulWidget {
  static const routeName = '/NewsPage';

  @override
  _NewsPageState createState() => _NewsPageState();
}

class _NewsPageState extends State<NewsPage> {
  NewsPageViewModel viewModel;

  RefreshController _refreshController =
      RefreshController(initialRefresh: false);

  void _onRefresh() async {
    // monitor network fetch
    await Future.delayed(Duration(milliseconds: 1000));
    setState(() {
      viewModel.getInformationData();
    });
    _refreshController.refreshCompleted();
  }

  void _onLoading() async {
    // monitor network fetch
    await Future.delayed(Duration(milliseconds: 1000));
    // if failed,use loadFailed(),if no data return,use LoadNodata()
    if (mounted) setState(() {});
    _refreshController.loadComplete();
  }

  @override
  Widget build(BuildContext context) {
    viewModel = Provider.of<NewsPageViewModel>(context);
    viewModel.getInformationData();

    return Scaffold(
        appBar: AppBar(
          elevation: 0,
          toolbarHeight: 120,
          title: SizedBox(
            width: double.infinity,
            child: Padding(
              padding: const EdgeInsets.only(left: 16, top: 24),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text("ข่าวประชาสัมพันธ์",
                      style: TextStyle(color: Colors.white, fontSize: 24)),
                  Padding(
                    padding: const EdgeInsets.only(top: 8.0, bottom: 16.0),
                    child: Text("แผนกฌาปนกิจสงเคราะห์\nกองสวัสดิการ",
                        style: TextStyle(color: Colors.white70, fontSize: 12)),
                  )
                ],
              ),
            ),
          ),
          backgroundColor: Color(0xFF7F3F97),
          leading: AppDrawer.appBarLeadIcon(context),
        ),
        drawer: AppDrawer().getAppDrawer(context),
        body: SafeArea(
          child: FutureBuilder<List<News>>(
              future: viewModel.newsList,
              builder: (context, snapshot) {
                if (snapshot.hasData) {
                  print('########## BUILD NEWS LIST ##########');

                  return NotificationListener<ScrollNotification>(
                      onNotification: (ScrollNotification scrollInfo) {
                        if (!viewModel.isLoadMore &&
                            scrollInfo.metrics.pixels ==
                                scrollInfo.metrics.maxScrollExtent) {
                          setState(() {
                            viewModel.isLoading = true;
                            viewModel.loadMoreData();
                          });
                        }

                        return true;
                      },
                      child: LoadingOverlay(
                        isLoading: viewModel.isLoading,
                        color: Colors.black54,
                        child: SmartRefresher(
                          enablePullDown: true,
                          header: WaterDropHeader(),
                          footer: CustomFooter(
                            builder: (BuildContext context, LoadStatus mode) {
                              Widget body;
                              if (mode == LoadStatus.idle) {
                                body = Text("pull up load");
                              } else if (mode == LoadStatus.loading) {
                                body = CupertinoActivityIndicator();
                              } else if (mode == LoadStatus.failed) {
                                body = Text("Load Failed!Click retry!");
                              } else if (mode == LoadStatus.canLoading) {
                                body = Text("release to load more");
                              } else {
                                body = Text("No more Data");
                              }
                              return Container(
                                height: 55.0,
                                child: Center(child: body),
                              );
                            },
                          ),
                          controller: _refreshController,
                          onRefresh: _onRefresh,
                          onLoading: _onLoading,
                          child: ListView.builder(
                              itemCount: snapshot.data.length,
                              itemBuilder: (context, index) {
                                return InkWell(
                                  child: ListTile(
                                    title: getItem(snapshot.data[index]),
                                  ),
                                  onTap: () {
                                    print(
                                        'Click on: ${snapshot.data[index].id}');
                                    setState(() {
                                      // viewModel.isLoading = true;
                                      _launchURL(context,
                                          snapshot.data[index].externalUrl);
                                    });
                                  },
                                );
                              }),
                        ),
                      ));
                } else {
                  return Container();
                }
              }),
        ));
  }

  Widget getItem(News news) {
    var date = DateFormat('yyyy-MM-dd').parse(news.displayDate);

    return Padding(
      padding: const EdgeInsets.all(24),
      child: Column(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Icon(Icons.person),
              Expanded(
                child: Padding(
                  padding: const EdgeInsets.only(left: 24.0),
                  child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Text('${news.createdBy}',
                            style: TextStyle(fontSize: 18)),
                        Text(DateFormat('dd MMM yy').format(date),
                            style: TextStyle(fontSize: 12)),
                      ]),
                ),
              ),
              Icon(
                Icons.keyboard_arrow_down,
                color: Colors.grey,
              )
            ],
          ),
          Padding(
            padding: const EdgeInsets.fromLTRB(8.0, 16.0, 8.0, 16.0),
            child: SizedBox(
                width: double.infinity,
                child: Text(news.informationCode,
                    style:
                        TextStyle(fontSize: 16, fontWeight: FontWeight.bold))),
          ),
          getImage(news),
          Padding(
            padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
            child:
                Html(data: news.detail.split("<\/strong><br \/><br \/>").last),
          ),
        ],
      ),
    );
  }

  Widget getImage(News news) {
    if (news.coverImage == null || news.coverImage.trim() == "") {
      return Container();
    } else {
      return Image(
        image: NetworkImage(news.coverImage),
      );
    }
  }

  void _launchURL(BuildContext context, String url) async {
    try {
      if (url.trim() == "") {
        return;
      }
      if (!url.startsWith("http")) {
        url = "http://" + url;
      }
      await launch(
        url,
        option: new CustomTabsOption(
          toolbarColor: Color(0xff8a56ac),
          enableDefaultShare: true,
          enableUrlBarHiding: true,
          showPageTitle: true,
          animation: new CustomTabsAnimation.slideIn(),
        ),
      );
    } catch (e) {
      // An exception is thrown if browser app is not installed on Android device.
      debugPrint(e.toString());
    }
  }
}
